PROGRAM_NAME='Main' 

(***********************************************************)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 03/07/2012  AT: 16:30:00        *)
(***********************************************************)
(* System Type : Rane HAL                                  *)
(***********************************************************)
(* REV HISTORY:                                            *)
(***********************************************************)
(*
    $History: $
*)
(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

dvTP = 10001:1:1
dvIPClient = 0:2:1
dvHAL = 0:FIRST_LOCAL_PORT:0

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

sRemoteIP = '10.0.0.6'
nRemotePort = 4996
nLocalPort = FIRST_LOCAL_PORT
nTCP = 1

BTN_DEFAULT_LEVEL = 3
BTN_MUTE_ALL = 4
BTN_WALL_OPEN = 5
BTN_CLASSICAL = 10
BTN_JAZZ = 11
BTN_REGGAE = 12
BTN_SOFTEN_VOCALS = 20
BTN_SMOOTH = 21
BTN_BOOST_BASS = 22
BTN_CONNECTED = 99
LVL_LOUNGE_VOLUME = 6

nMaxArrayLen = 256

(* command buttons *)
INTEGER nCommandChannels[] = { BTN_DEFAULT_LEVEL }

(* toggle buttons *)
INTEGER nToggleChannels[] = { BTN_MUTE_ALL, BTN_WALL_OPEN }

(* level sliders *)
INTEGER nLevelChannels[] = { LVL_LOUNGE_VOLUME }

(* radio buttons *)
nSourceSelNumber = 1
INTEGER nSourceSelChannels[] = { BTN_CLASSICAL, BTN_JAZZ, BTN_REGGAE }
nToneSelNumber = 2
INTEGER nToneSelChannels[] = { BTN_SOFTEN_VOCALS, BTN_SMOOTH, BTN_BOOST_BASS }

(***********************************************************)
(*              DATA TYPE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

cData[1024]

bConnected = false

nCommandIndex 

nToggleIndex
INTEGER bToggleValues[nMaxArrayLen]

nLevelIndex
INTEGER nLevelValues[nMaxArrayLen]

nSelectorNumber
nChannelIndex

nSourceSelValue
INTEGER bSourceSelRadios[nMaxArrayLen]

nToneSelValue
INTEGER bToneSelRadios[nMaxArrayLen]

(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)
(* EXAMPLE: DEFINE_FUNCTION <RETURN_TYPE> <NAME> (<PARAMETERS>) *)
(* EXAMPLE: DEFINE_CALL '<NAME>' (<PARAMETERS>) *)
DEFINE_FUNCTION INTEGER FindToggleIndex(INTEGER channel)
{
    stack_var INTEGER i
    for (i = 1; i <= length_array(nToggleChannels); i++)
    {
      if (nToggleChannels[i] == channel)
      {
        return i
      }
    }
    
    return 0
}

DEFINE_FUNCTION INTEGER FindLevelIndex(INTEGER channel)
{
    stack_var INTEGER i
    for (i = 1; i <= length_array(nLevelChannels); i++)
    {
      if (nLevelChannels[i] == channel)
      {
        return i
      }
    }
    
    return 0
}

DEFINE_FUNCTION UpdateSourceSelectorRadios(INTEGER channel)
{
    stack_var INTEGER i
    for (i = 1; i <= length_array(nSourceSelChannels); i++)
    {
        if (nSourceSelChannels[i] == channel)
	{
	    bSourceSelRadios[i] = true
	}
	else
	{
	    bSourceSelRadios[i] = false
	}
    }
}

DEFINE_FUNCTION UpdateToneSelectorRadios(INTEGER channel)
{
    stack_var INTEGER i
    for (i = 1; i <= length_array(nToneSelChannels); i++)
    {
        if (nToneSelChannels[i] == channel)
	{
	    bToneSelRadios[i] = true
	}
	else
	{
	    bToneSelRadios[i] = false
	}
    }
}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

set_length_array(bToggleValues, length_array(nToggleChannels))
set_length_array(nLevelValues, length_array(nLevelChannels))
set_length_array(bSourceSelRadios, length_array(nSourceSelChannels))
set_length_array(bToneSelRadios, length_array(nToneSelChannels))

ip_client_open(nLocalPort, sRemoteIP, nRemotePort, nTCP)

(***********************************************************)
(*                THE EVENTS GO BELOW                      *)
(***********************************************************)
DEFINE_EVENT

DATA_EVENT[dvHAL]
{
  ONLINE:
  {    
    stack_var INTEGER i
    
    bConnected = true
    
    for (i = 1; i <= length_array(nToggleChannels); i++)
    {
        Send_String dvHAL, "'<T&', itoa(nToggleChannels[i]), '>'"
    }
    
    for (i = 1; i <= length_array(nLevelChannels); i++)
    {
        Send_String dvHAL, "'<L&', itoa(nLevelChannels[i]), '>'"
    }
    
    Send_String dvHAL, "'<S&', itoa(nSourceSelNumber), '>'"
    Send_String dvHAL, "'<S&', itoa(nToneSelNumber), '>'"
  }
  OFFLINE:
  {
    bConnected = false
    ip_client_close(nLocalPort)
  }
  STRING:
  {
    cData = DATA.TEXT
    
    select
    {
      active(find_string(cData, '<T&', 1)):
      {
      	remove_string(cData, '<T&', 1)
	nToggleIndex = FindToggleIndex(atoi(cData))
	remove_string(cData, '&', 1)
	if (nToggleIndex > 0 && nToggleIndex <= length_array(nToggleChannels))
	{
	    bToggleValues[nToggleIndex] = atoi(cData)
	}
	remove_string(cData, '>', 1)
      }
      active(find_string(cData, '<L&', 1)):
      {
	remove_string(cData, '<L&', 1)
	nLevelIndex = FindLevelIndex(atoi(cData))
	remove_string(cData, '&', 1)
	if (nLevelIndex > 0 && nLevelIndex <= length_array(nLevelChannels))
	{
	    nLevelValues[nLevelIndex] = atoi(cData) * 255/1000
	    send_level dvTP, nLevelChannels[nLevelIndex], nLevelValues[nLevelIndex]
	}
	remove_string(cData, '>', 1)
      }
      active(find_string(cData, '<S&', 1)):
      {
	remove_string(cData, '<S&', 1)
	nSelectorNumber = atoi(cData)
	remove_string(cData, '&', 1)
	if (nSelectorNumber == nSourceSelNumber)
	{
	    nSourceSelValue = atoi(cData)
	    UpdateSourceSelectorRadios(nSourceSelChannels[nSourceSelValue + 1])
	}
	else if (nSelectorNumber == nToneSelNumber)
	{
	    nToneSelValue = atoi(cData)
	    UpdateToneSelectorRadios(nToneSelChannels[nToneSelValue + 1])	    
	}
	remove_string(cData, '>', 1)
      }
    }
    
    DATA.TEXT = ''
    cData = ''
  }
}

BUTTON_EVENT[dvTP, nCommandChannels]
{
PUSH:
    {
        nCommandIndex = GET_LAST(nCommandChannels)
	if (nCommandIndex > 0 && nCommandIndex <= length_array(nCommandChannels))
	{
	    Send_String dvHAL, "'<C&', itoa(nCommandChannels[nCommandIndex]), '>'"
	}
    }
}

BUTTON_EVENT[dvTP, nToggleChannels]
{
RELEASE:
    {
        nToggleIndex = GET_LAST(nToggleChannels)
	if (nToggleIndex > 0 && nToggleIndex <= length_array(nToggleChannels))
	{
	    bToggleValues[nToggleIndex] = !bToggleValues[nToggleIndex]
	    Send_String dvHAL, "'<T&', itoa(nToggleChannels[nToggleIndex]), '&', itoa(bToggleValues[nToggleIndex]), '>'"
	}    
    }
}

BUTTON_EVENT[dvTP, nSourceSelChannels]
{
PUSH:
    {
	nChannelIndex = GET_LAST(nSourceSelChannels)
	if (nChannelIndex > 0 && nChannelIndex <= length_array(nSourceSelChannels))
	{
	    nSourceSelValue = nChannelIndex - 1
	    UpdateSourceSelectorRadios(nSourceSelChannels[nChannelIndex])
	    Send_String dvHAL, "'<S&', itoa(nSourceSelNumber), '&', itoa(nSourceSelValue), '>'"
	} 
    }
}

BUTTON_EVENT[dvTP, nToneSelChannels]
{
PUSH:
    {
	nChannelIndex = GET_LAST(nToneSelChannels)
	if (nChannelIndex > 0 && nChannelIndex <= length_array(nToneSelChannels))
	{
	    nToneSelValue = nChannelIndex - 1
	    UpdateToneSelectorRadios(nToneSelChannels[nChannelIndex])
	    Send_String dvHAL, "'<S&', itoa(nToneSelNumber), '&', itoa(nToneSelValue), '>'"
	} 
    }
}

LEVEL_EVENT[dvTP, nLevelChannels]
{
    nLevelIndex = GET_LAST(nLevelChannels)
    if (nLevelIndex > 0 && nLevelIndex <= length_array(nLevelChannels))
    {
        if (LEVEL.VALUE <> nLevelValues[nLevelIndex])
	{
	    nLevelValues[nLevelIndex] = LEVEL.VALUE
	    Send_String dvHAL, "'<L&', itoa(nLevelChannels[nLevelIndex]), '&', itoa(nLevelValues[nLevelIndex] * 1000/255), '>'"
	}         	
    }
}

(***********************************************************)
(*            THE ACTUAL PROGRAM GOES BELOW                *)
(***********************************************************)
DEFINE_PROGRAM

[dvTP, BTN_CONNECTED] = bConnected

if (!bConnected)
{
    ip_client_open(nLocalPort, sRemoteIP, nRemotePort, nTCP)
}
else
{
    for (nToggleIndex = 1; nToggleIndex <= length_array(nToggleChannels); nToggleIndex++)
    {
	[dvTP, nToggleChannels[nToggleIndex]] = bToggleValues[nToggleIndex]
    }

    for (nChannelIndex = 1; nChannelIndex <= length_array(nSourceSelChannels); nChannelIndex++)
    {
	[dvTP, nSourceSelChannels[nChannelIndex]] = bSourceSelRadios[nChannelIndex]
    }

    for (nChannelIndex = 1; nChannelIndex <= length_array(nToneSelChannels); nChannelIndex++)
    {
	[dvTP, nToneSelChannels[nChannelIndex]] = bToneSelRadios[nChannelIndex]
    }     
}

(***********************************************************)
(*                     END OF PROGRAM                      *)
(*        DO NOT PUT ANY CODE BELOW THIS COMMENT           *)
(***********************************************************)
